#include <qwidget.h>
#include <qcursor.h>
#include <qpainter.h>
#include <stdio.h>
#include <stdlib.h>
#include "qgmap.h"
#include "gmap.h"

#include <qdialog.h>
#include <qdatetime.h>

#include "pos.h"
#include "inputdialog.h"

GMap::GMap( QWidget *parent, const char *name, WFlags fl)
	: QWidget( parent, name, fl), img_cache(31,31)
{
//#ifndef QWS
	resize(640,640);
//#endif

	is_initialized = 0;
	is_map_click = FALSE;
	is_scale_click = FALSE;

	popup_menu.insertItem("Map Path", this, SLOT(menu_activated(int)));

	map_path = conf.readEntry("MapPath", DEFAULT_MAP_PATH);
//	printf("MapPath = %s\n", map_path.ascii());

	menu_pic.load(MENUIMG_FILE);

	// 8MB Cache
	// 256x256x32bpp = 256kB/Picture
	// 8MB/256kB = 32 Picture
	cache.setCacheLimit(8192);

	img_cache.setAutoDelete(TRUE);

	scale = new Scale(&scr, &center);
	calcScale();
}

void GMap::updateCenter(bool force)
{
	int widget_top, widget_left;
	int new_ltop, new_lleft;

	// ウィジェットの左上部の地図座標
	widget_top = center.getY()-this->height()/2;
	widget_left = center.getX()-this->width()/2;
//	printf("widget_left=%d, widget_top=%d\n", widget_left, widget_top);

	// 読み込み画像の左上の地図座標
	new_ltop = widget_top & ~(IMAGE_SIZE-1);
	new_lleft = widget_left & ~(IMAGE_SIZE-1);
//	printf("load_left=%d, load_top=%d\n", load_left, load_top);
//	printf("new_lleft=%d, new_ltop=%d force=%d\n", new_lleft, new_ltop, force);

	Pos pos(new_lleft, new_ltop, center.getZoom());

//	QTime t;
	// 強制読み込み時、スクロールしたときロード時
	if(load_left != new_lleft || load_top != new_ltop || force){

//		t.start();
		//全画像ロード
		loadMap(pos);
		//内部で load_left, load_top を更新
//		printf("forceLoadMap() %dms.\n", t.elapsed());
	}

	// 再描画指示(消去しない)
	repaint(FALSE);
}

void GMap::paintEvent( QPaintEvent *e )
{
	int x,y;
	int dx, dy;

	if(!is_initialized){
		int i;

		is_initialized = 1;

		scr.resize(this->width(), this->height());

		x_num = (this->width()/IMAGE_SIZE)+2;
		y_num = (this->height()/IMAGE_SIZE)+2;

		pix.resize(x_num*y_num);

		for(i=0;i<x_num*y_num;i++){
			pix[i] = new Map(&cache, &img_cache, &map_path);
		}

		updateCenter(TRUE);
	}

	/*
	  if ( !e->rect().intersects( cannonRect() ) )
	  return;

	  QRect cr = cannonRect();
	  QPixmap pix( cr.size() );
	*/

	dx = (center.getX() - this->width()/2) - load_left;
	dy = (center.getY() - this->height()/2) - load_top;

#if 0
	printf("GMap::paintEvent load_left=%d, load_top=%d\n", load_left, load_top);
	printf("GMap::paintEvent widget_x=%d, widget_y=%d\n",
		   (center.getX() - this->width()/2), (center.getY() - this->height()/2));
	printf("GMap::paintEvent width=%d, height=%d\n", this->width(), this->height());
	printf("GMap::paintEvent dx=%d, dy=%d\n", dx, dy);
#endif

	QPixmap pixmap(e->rect().size());
	QPainter p( &scr );
	p.setPen(Qt::black);

	for(y = 0; y < y_num; y++){
		for(x = 0; x < x_num; x++){
			p.drawPixmap( x*IMAGE_SIZE-dx, y*IMAGE_SIZE-dy, pix[y*x_num+x]->getPix(),
						  0, 0, IMAGE_SIZE, IMAGE_SIZE);
//			p.drawRect( x*IMAGE_SIZE-dx, y*IMAGE_SIZE-dy, IMAGE_SIZE+1, IMAGE_SIZE+1);
		}
	}

	p.drawPixmap( 10, 10, menu_pic);
	p.end();

	scale->paintEvent( e );
	drawScale(&scr);

	p.begin( this );
	p.drawPixmap( 0, 0, scr );

}

void GMap::wheelEvent( QWheelEvent *e )
{
	zoomup(e->delta()<0?1:-1);
}

void GMap::mouseMoveEvent( QMouseEvent *e )
{
	if(is_map_click){
		QPoint pos = click_pos - e->pos();

		center = center.offset(pos.x(), pos.y());
		center.normalize();
		updateCenter(FALSE);

		click_pos = e->pos();
	}
	if(is_scale_click){
		if(scale->moveEvent(e)){
			calcScale();
			updateCenter(TRUE);
		}
	}
}

void GMap::mousePressEvent( QMouseEvent *e )
{
	QRect menu_rect(10,10,16,16);

	if(e->button() != LeftButton)
		return;

	if(!is_map_click && !is_scale_click){
		if(menu_rect.contains(e->pos())){
//			printf("click menu\n");
			popup_menu.popup(QCursor::pos());
			return;
		}else if(scale->rect().contains(e->pos())){
			if(scale->pressEvent(e)){
				calcScale();
				updateCenter(TRUE);
			}
			is_scale_click = TRUE;
			return;
		}
	}

	is_map_click = TRUE;
	click_pos = e->pos();
}

void GMap::mouseReleaseEvent( QMouseEvent *e )
{
	if(e->button() != LeftButton)
		return;

	if(is_scale_click){
		if(scale->releaseEvent(e)){
			calcScale();
			updateCenter(TRUE);
		}
		is_scale_click = FALSE;
		return;
	}

	is_map_click = FALSE;
	calcScale();
	repaint(FALSE);
}

void GMap::keyPressEvent ( QKeyEvent * e )
{
	int shift = (e->state() & ShiftButton)?SHIFT_MUL:1;

	switch(e->key()){
	case Key_Left:
		scroll(-KEY_SCROLL*shift, 0);
		break;
	case Key_Right:
		scroll(KEY_SCROLL*shift, 0);
		break;
	case Key_Up:
		scroll(0, -KEY_SCROLL*shift);
		break;
	case Key_Down:
		scroll(0, KEY_SCROLL*shift);
		break;
	case Key_I:
		zoomup(-1);
		break;
	case Key_O:
		zoomup(1);
		break;
	}
}

void GMap::keyReleaseEvent ( QKeyEvent * e )
{
	calcScale();
}

/* 全画像の読み込み */
void GMap::loadMap(Pos loadpos)
{
	int x, y;

	for(y = 0; y < y_num; y++){
		for(x = 0; x < x_num; x++){
			pix[y*x_num+x]->loadPixmap(loadpos.offset(x*IMAGE_SIZE,y*IMAGE_SIZE));
		}
	}
	load_left = loadpos.getX();
	load_top = loadpos.getY();
}

void GMap::scroll(int x, int y)
{
	center = center.offset(x, y);
	updateCenter(FALSE);
}

void GMap::zoomup(int z)
{
	center.zoomup(z);
	calcScale();
	updateCenter(TRUE);
}

void GMap::menu_activated(int id)
{
//	printf("menu %d\n",id);

	InputDialog dlg(this, "Map Path", TRUE);

	dlg.setText(conf.readEntry("MapPath", DEFAULT_MAP_PATH));

	int ret = dlg.exec();
//	printf("ret:%d text=%s\n", ret, dlg.getText().ascii());
	if(ret == QDialog::Accepted){
		map_path = dlg.getText();
		conf.writeEntry("MapPath", map_path);
//		printf("map_path=%s\n", map_path.ascii());
		updateCenter(TRUE);
	}
}

void GMap::calcScale()
{
	scale_size = center.getScaleSize(SCALE_MAX_SIZE, &scale_mul);
}

void GMap::drawScale(QPixmap *scr)
{
	QPainter p(scr);

	int xoff = 10, yoff = scr->height() - 24;

	p.setPen(Qt::white);
	p.drawRect(xoff, yoff+4, 4,14);
	p.drawRect(xoff+scale_size, yoff+4, 4,14);
	p.drawRect(xoff, yoff+4+10, scale_size+4,4);

	p.setPen(Qt::black);
	p.drawRect(xoff+1, yoff+4+1, 2,10);
	p.drawRect(xoff+1+scale_size, yoff+4+1, 2,10);
	p.drawRect(xoff+1, yoff+4+11, scale_size+2,2);

	QFont font = QFont::defaultFont();
	font.setPixelSize(SCALE_FONT_SIZE);
	p.setFont(font);
	QString str;
	if(scale_mul>1000)
		str.sprintf("%dkm",scale_mul/1000);
	else
		str.sprintf("%dm",scale_mul);

	p.drawText(xoff+10, yoff+10, str);
}
